local addon, data = ...

-- Creates a distance to unit frame

local gadgetIndex = 0

local playerFrame = WT.UnitFrame:Create("player")
local distances = {}

local selUnitToTrack = nil
local distanceTxtColor = nil
local labTextColor = nil
local labFontSize = nil
local sldFontSize = nil
local chkOverride50 = nil
local labOverride50 = nil

local dist_context = nil

local function OnNameChange(distance, name)
	if not name then
		distance.txtRange:SetVisible(false)
		distance.txtName:SetVisible(false)
		distance.background:SetVisible(false)
	else
		distance.txtRange:SetVisible(true)
		distance.txtName:SetVisible(true)
		distance.background:SetVisible(true)
		local nm = name:upper()
		if nm:len() > 25 then nm = nm:sub(1, 25) end
		distance.txtName:SetText(nm)
	end
end

local function OnCoordChange(distance, unitCoord)
	if not WT.Player then return end
	if not WT.Player.coord then return end
	if not unitCoord then return end

	if WT.Player.coord and unitCoord then
		local range = WT.Utility.MeasureDistance(
			WT.Player.coord[1], WT.Player.coord[2], WT.Player.coord[3],
			unitCoord[1], unitCoord[2], unitCoord[3])  
		distance.txtRange:SetLabelText(string.format("%.01f", range) .. "m")
		if distance.config.ovr50 then
			if range > 50 then
				distance.txtRange:SetFontColor(1,0,0)
			else
				distance.txtRange:SetFontColor(unpack(distance.config.txtcol))
			end
		end
	else
		distance.txtRange:SetLabelText("--")
	end
end

local function OnPlayerCoordChange(distance, playerCoord)
	for idx,distance in ipairs(distances) do
		distance:ApplyBindings()		
	end
end

local function Create(configuration)
	local rfHeight = 70

	local distance = WT.UnitFrame:Create(configuration.unitSpec)
	
	if dist_context == nil then
		dist_context = UI.CreateContext(addon.identifier)
	end
	
	distance:SetParent(dist_context)
	distance:SetWidth(60)

	local txtRange = distance:CreateElement({
		id="txtRange", type="Label", parent=distance, layer=1000,
		attach = {{ point="TOPLEFT", element=distance, targetPoint="TOPLEFT", offsetX=0, offsetY=0 }},
		visibilityBinding="name", text="--", default="", fontSize=configuration.fontsize or 12, outline=false,
		color = { r=configuration.txtcol[1], g=configuration.txtcol[2], b=configuration.txtcol[3], a=configuration.txtcol[4] },
	});

	distance:SetHeight(txtRange:GetHeight())
	distance.txtRange = txtRange
	
	distance:CreateBinding("coord", distance, OnCoordChange, nil)
	playerFrame:CreateBinding("coord", playerFrame, OnPlayerCoordChange, nil)
	
	table.insert(distances, distance)
	
	distance:SetLayer(90)
	txtRange:SetLayer(95)
	
	distance.config = configuration
	return distance
end


local dialog = false

local function ConfigDialog(container)	

	local frmConfig = UI.CreateFrame("Frame", "dsConfig", container)
	frmConfig:SetPoint("TOPLEFT", container, "TOPLEFT")
	frmConfig:SetPoint("BOTTOMRIGHT", container, "BOTTOMRIGHT", 0, -32)
	
	selUnitToTrack = WT.Control.Select.Create(frmConfig, "Unit to track:", "player.target",
			{
				{text="Target", value="player.target"},
				{text="Target's Target", value="player.target.target"},
				{text="Focus", value="focus"},
				{text="Focus's Target", value="focus.target"},
				{text="Group Member 1.1", value="group01" },
				{text="Group Member 1.2", value="group02" },
				{text="Group Member 1.3", value="group03" },
				{text="Group Member 1.4", value="group04" },
				{text="Group Member 1.5", value="group05" },
				{text="Group Member 2.1", value="group06" },
				{text="Group Member 2.2", value="group07" },
				{text="Group Member 2.3", value="group08" },
				{text="Group Member 2.4", value="group09" },
				{text="Group Member 2.5", value="group10" },
				{text="Group Member 3.1", value="group11" },
				{text="Group Member 3.2", value="group12" },
				{text="Group Member 3.3", value="group13" },
				{text="Group Member 3.4", value="group14" },
				{text="Group Member 3.5", value="group15" },
				{text="Group Member 4.1", value="group16" },
				{text="Group Member 4.2", value="group17" },
				{text="Group Member 4.3", value="group18" },
				{text="Group Member 4.4", value="group19" },
				{text="Group Member 4.5", value="group20" },
			})

	labTextColor = UI.CreateFrame("Text", "labTextColor", frmConfig)
	labTextColor:SetText("Text color")

	labFontSize = UI.CreateFrame("Text", "labFontSize", frmConfig)
	labFontSize:SetText("Font size")
	
	distanceTxtColor = WT.Control.ColorPicker.Create(frmConfig, "Text Color", 0.9, 0.9, 0.9, 1)

	selUnitToTrack:SetPoint("TOPLEFT", frmConfig, "TOPLEFT", 0,0)
	labTextColor:SetPoint("TOPLEFT", selUnitToTrack, "BOTTOMLEFT", 0, 6)
	distanceTxtColor:SetPoint("CENTERLEFT", labTextColor, "CENTERLEFT", 80, 0)
	
	labFontSize:SetPoint("TOPLEFT", labTextColor, "BOTTOMLEFT", 0, 6)
	
	sldFontSize = UI.CreateFrame("SimpleSlider", "sldFontSize", frmConfig)
	sldFontSize:SetRange(6,24)
	sldFontSize:SetPosition(12, true)
	
	sldFontSize:SetPoint("CENTERLEFT", labFontSize, "CENTERLEFT", 80, 0)
	
	chkOverride50 = UI.CreateFrame("SimpleCheckbox", "chkOverride50", frmConfig)
	chkOverride50:SetText("Override color when range >50m")
	chkOverride50:SetChecked(false)
	chkOverride50:SetPoint("TOPLEFT", distanceTxtColor, "TOPRIGHT", 6, 0)
end

local function GetConfiguration()
	local config = {}
	
	config.unitSpec = selUnitToTrack:GetText()
	config.txtcol = { distanceTxtColor:GetColor() }
	config.fontsize = sldFontSize:GetPosition()
	config.ovr50 = chkOverride50:GetChecked()
	
	return config
end

local function SetConfiguration(config)
	selUnitToTrack:SetText(config.unitSpec)
	distanceTxtColor:SetColor(unpack(config.txtcol))
	sldFontSize:SetPosition(config.fontsize or 12, true)
	chkOverride50:SetChecked(config.ovr50 or false)
end


WT.Gadget.RegisterFactory("Distance",
	{
		name="Distance",
		description="Distance",
		author="Adelea@Icewatch-EU",
		version="1.10r1",
		iconTexAddon=addon.identifier,
		iconTexFile="img/TapeMeasure.png",
		["Create"] = Create,
		["ConfigDialog"] = ConfigDialog,
		["GetConfiguration"] = GetConfiguration, 
		["SetConfiguration"] = SetConfiguration, 
	})

